/* This program was written with lots of love under the GPL by Jonathan
 * Blandford <jrb@gnome.org>
 */

#include <gtk/gtk.h>
#include <shell/cc-panel.h>

void gnome_keybinding_properties_init (CcPanel *panel, GtkBuilder *builder);
void gnome_keybinding_properties_dispose (CcPanel *panel);
